/*********************************************************************************/
/* Name: chf_usefull_lcd.c														 */
/*	    																		 */
/* Author: Christof Leeb													     */
/*																				 */
/* Verwendungszeck: initialisiert das LCD und Routinen zur Anzeige               */
/*********************************************************************************/
#include "main.h"

void init_display(void)
{
	/// ---------- Activating the LCD-Modul (HD44780) with C167 Databus) ---------- 
	///
	///		Interfaces:		D0...D7: Data transmission line							 
	///                     A1     : register select (0=instruction,1=data)			 
	///						WRL    : write enable (low byte)						 
	///						CS3    : chip select									 
	unsigned char x;

	/* BUSCON3 / ADDRSEL3 */
 	BUSCON3 = 0x0430;	/* 0: 15 waitstates
						   3: RWD=out, MTT=no waitstates
						      external bus configuration
							  8-bit Demultiplexed Bus
						   4: normale ALE signal
							  external bus enabled within resprective address window(ADDRSEL)
						   0: external bus cycle is controlled by bit fild MTCT only
							  cs_ signal is independent of the read command RD_ (RD_ = active low)
							  cs_ signal is independent of the write comd. WR_,WRL_,WRH_		

 	/* Addressarea = 80:0000 ... 80:0FFF */
	ADDRSEL3 = 0x8000;

	/* Initiation Timer T6 - 1.6 us */
 	T6CON = 0x0002;

	/* Initiation Software */
	x = 0x38;
	wait(10000); // 16 ms
	_LCD_CNTL = x;							//00111000=x  RS=0,RW=0,DB7=0,DB6=0,DB5=1,DB4=1;DB3-DB0=..
 	wait(3000);  // 4.8 ms
	_LCD_CNTL = x; 
 	wait(3000);  // 4.8 ms
 	_LCD_CNTL = x;
 	wait(3000);  // 4.8 ms

	/* init display */
 	_LCD_CNTL = x;
 	wait(3000);  // 4.8 ms
}

void wait(unsigned int w)
{
	/* breakup T6: 1.6 microseconds */
 	T6 = 0;
 	T6R = 1;
 	while (T6 < w);
 	T6R = 0;
}

void write_string(char * s, unsigned char line)
{
	unsigned int i;

  	/* set DDRAM Address */
 
  	switch (line)
  	{
  		case 1: _LCD_CNTL = 0x80; break;
  		case 2: _LCD_CNTL = 0xc0; break;
  		case 3: _LCD_CNTL = 0x90; break;
  		case 4: _LCD_CNTL = 0xD0; break;
		case 5: _LCD_CNTL = 0x94; break;
  	}
 
  	wait(3000);  // 4.8 ms
 	for (i=0; i<strlen(s); i++)
  	{
    	_LCD_DATA = s[i];
    	wait(30);  // 48 us
  	}
}

void clear_display(void)
{	
	/* set display off */
 	_LCD_CNTL = 0x08;
 	wait(3000);  // 4.8 ms

 	/* clear display */
	_LCD_CNTL = 0x01;
 	wait(3000);  // 4.8 ms

 	/* display on */
 	_LCD_CNTL = 0x0C;
 	wait(3000);  // 4.8 ms

 	/* entry mode set */
   	_LCD_CNTL = 0x06;
   	wait(3000);  // 4.8 ms
}
