/*********************************************************************************/
/* Name: init.c						  											 */
/*	    																		 */
/* Author: Huber Michael													     */
/*																				 */
/* Verwendungszeck: intialisierung der Timer  									 */
/*********************************************************************************/
#include "main.h"

/*Initialisation***************************************************/

void Project_Init(void)
{  
  	/// ------------ Serial Port ----------
  	///  8-bit data asynchronous operation
  	///  one stop bit
  	///  ignore framing error
  	///  ignore overrun error
  	///  divide clock by reload value + constant
  	///  receiver enabled
  	/*S0CON  = 0x0011;

  	//   ASC baudrate reload register
  	///  baudrate =  9615,38 Baud
  	S0BG   = 0x0040;

  	///  enable receive interrupt
  	///  receive interrupt priority level(ILVL) = 8
  	///  receive interrupt group level (GLVL) = 0
  	S0RIC = 0x0060;
  
  	///  set Transmission Interrupt Flag (Buffer empty)
  	S0TIC = 0x80;
  
  	P3    |= 0x0400;    //  set P3.10 output latch (TXD0)
  	DP3   |= 0x0400;    /// set P3.10 direction control (TXD0 output)
  	DP3   &= 0xF7FF;    /// reset P3.11 direction control (RXD0 input)

  	S0R    = 1;         //  enable baudrate generator  
  	*/
    
  	/******************************************************************/
  	/// ------------ Timer 3 Control Register ----------
  	///  timer 3 works in timer mode
  	///  prescaler factor is 1024
  	///  timer 3 run bit is reset
  	///  up/down control bit is set
  	///  external up/down control is disabled
  	///  alternate output (toggle T3OUT) function is disabled 
  	T3CON = 0x0487;
  	T3    = 0x4C4B;  //  load timer 3 register
  
  	/// ---------- Timer 2 Control Register ----------
  	///  timer 2 works in counter mode
  	///  any transition (rising and falling edge) of output toggle latch T3OTL
  	///  timer/counter 2 run bit is set
  	///  up/down control bit is reset 
  	///  external up/down control is disabled
  	T2CON = 0x00CF;
  	T2    = 0x0005;  //  load timer 2 register

  	///  enable timer 2 interrupt
  	///  timer 2 interrupt priority level(ILVL) = 4
  	///  timer 2 interrupt group level (GLVL) = 0
  	T2IC = 0x0050;

  	/// ---------- Timer 4 Control Register ----------
  	///  timer 4 works in reload mode 
  	///  timer T4 is disabled
  	T4CON = 0x0020;
  	T4    = 0x4C4B;  //  load timer 4 register

  	IEN = 1;
  
  	/******************************************************************/ 
  	/// ---------- LCD und Taster ----------
  	//init_taster();		/* Aufruf der Taster Funktion  	*/
  	init_T6();
  	IO();				/* Aufruf der IO Funktion 		*/
  	init_display();
  	init_relais();
}
/******************************************************************/