/*********************************************************************************/
/* Name: mic_usefull_ds1820.c													 */
/*	    																		 */
/* Author: Michael Huber (nur editiert)											 */
/*																				 */
/* Verwendungszeck: initialisierung von Port 2, Grundroutinen fr DS1820, 		 */
/*					Temperaturmessung											 */
/*********************************************************************************/
#include "main.h"

#define DS1820  P2         	// P2.0 output port line  bit 
#define DSMSK1 0x0001		// mask P2.0 @ DS1820_IN 
#define DSMSK2 0x0002		// mask P2.1 @ DS1820_IN 
#define DSMSK3 0x0004		// mask P2.2 @ DS1820_IN 
#define DS1820_DIR 	DP2		// port line direction bit 
#define ODIR 0x0007			// with |= DP2.0, DP2.1 and DP2.2 Output 
#define IDIR 0xfff8			// with &= DP2.0, DP2.1 and DP2.2 Input 
#define HIGH 0x0007			// use with |= ! 
#define LOW  0xfff8			// use with &= ! 


static unsigned char scp1[]={0, 0, 0, 0, 0, 0, 0, 0, 0};
static unsigned char scp2[]={0, 0, 0, 0, 0, 0, 0, 0, 0};
static unsigned char scp3[]={0, 0, 0, 0, 0, 0, 0, 0, 0};

unsigned int temp_1=0;
unsigned int temp_2=0;
unsigned int temp_3=0;


static void wait5us()
{
  	int i;
  	for (i=9; i>0; i--);
}               // eof wait5us 
/* ============================================================ */

static void wait80us()
{
  	int i;
  	for (i=380; i>0; i--);
}               // eof wait80us 
/* ============================================================ */

static void wait500us()
{
  	int i;
  	for (i=2500; i>0; i--);
}               // eof wait500us 
/* ============================================================ */

// set dallas ds1820 into a known state
static void reset()
{
	// output: high, min. 480us low, high, answer (ignored):60-240us low
  	DS1820|=HIGH;
  	DS1820_DIR|=ODIR; // initial state 
  	wait80us();
	// send reset command
  	DS1820&=LOW;
  	wait500us();
  	DS1820|=HIGH;
  	wait500us(); // ignore answer 
}               // eof reset 
/* ============================================================= */


/* ============================================================= */
// additional routines for read of dallas-temperature:

// output the skip rom command (DS1820)
static void wr_srcmd()
{
	unsigned char cmd = 0xCC;
  	int i;
   	// 80us high == sync, write bit: bit==1: 5us low, 80us high
   	//  bit==0:  85us low, high
  	DS1820_DIR|=ODIR;
  	for (i=0; i<8; i++, cmd>>=1)
  	{
    	DS1820|=HIGH;
    	wait80us(); // sync 
    	DS1820&=LOW;
    	wait5us();
    	if (cmd&0x01) DS1820|=HIGH; // write bit 
    	wait80us();
    	DS1820|=HIGH;
  	}
}               // eof wr_srcmd 
/* ============================================================ */

// output the convert temperature command (DS1820)
static void wr_ctcmd()
{
  	unsigned char cmd = 0x44;
  	int i;
   	// 2us high == sync, write bit: bit==1: 5us low, 80us high
   	//  bit==0:  85us low, high
  	DS1820_DIR|=ODIR;
  	for (i=0; i<8; i++, cmd>>=1)
  	{
    	DS1820|=HIGH;
    	wait5us(); // sync 
    	DS1820&=LOW;
    	wait5us();
    	if (cmd&0x01) DS1820|=HIGH; // write bit 
      	wait80us();
    	DS1820|=HIGH;
  	}
  	DS1820_DIR&=IDIR;
}               // eof wr_ctcmd 
/* ============================================================ */

// output the read scratchpad command (DS1820)
static void wr_rscmd()
{
  	unsigned char cmd = 0xBE;
  	int i;
   	// 2us high == sync, write bit: bit==1: 5us low, 80us high
   	//  bit==0:  85us low, high
  	DS1820_DIR|=ODIR;
  	for (i=0; i<8; i++, cmd>>=1)
  	{
    	DS1820|=HIGH;
    	wait5us(); // sync 
    	DS1820&=LOW;
    	wait5us();
    	if (cmd&0x01) DS1820|=HIGH; // write bit 
		wait80us();
    	DS1820|=HIGH;
  }
}               // eof wr_rscmd 
/* ============================================================ */

// generate read time slots and read scratchpad
static void rd_scp()
{
  	int i, j;
  	unsigned char *b, *c, *d;

  	for (i=0; i<9; i++) // 9 bytes 
  	{
    	b=&scp1[i];
    	c=&scp2[i];
		d=&scp3[i];
		for (j=0; j<8; j++) // 8 bits/byte 
    	{
      		*b>>=1;
	  		*c>>=1;
	  		*d>>=1;	
      		DS1820_DIR|=ODIR;
      		DS1820|=HIGH;
      		wait5us(); // sync
      		DS1820&=LOW;
      		wait5us();
      		DS1820_DIR&=IDIR;
      		wait5us();
      		if (DS1820&DSMSK1) *b|=0x80;
	  		if (DS1820&DSMSK2) *c|=0x80;
	  		if (DS1820&DSMSK3) *d|=0x80;
      		wait80us();
    	}
  	}
}               // eof rd_scp 
/* ============================================================ */

void rd_temp()
{
  	reset();
  	wr_srcmd();// Skip Rom 
  	wait5us();
  	wr_ctcmd(); // Convert Temperature 
  	wait5us();
  	wait80us(); wait80us();// Wait till end of conversion 
  	reset();
  	wr_srcmd(); // Skip Rom 
  	wait5us();
  	wr_rscmd();	// Read Scratchpad Command
  	wait5us();
  	rd_scp(); // Read Scratchpad
  
  	if (min_count>1) //die erste Messung wird uebersprungen -> Sensor aklimatisieren
	{
    	temp_1 = (((int)scp1[0] + (int)scp1[1] * 256) * 50)-((((unsigned int)scp1[7] * 25 - (unsigned int)scp1[6] * 25) / scp1[7]));
  		temp_2 = (((int)scp2[0] + (int)scp2[1] * 256) * 50)-((((unsigned int)scp2[7] * 25 - (unsigned int)scp2[6] * 25) / scp2[7]));
  		temp_3 = (((int)scp3[0] + (int)scp3[1] * 256) * 50)-((((unsigned int)scp3[7] * 25 - (unsigned int)scp3[6] * 25) / scp3[7]));
  	}
	else
	{
		temp_1=0;
		temp_2=0;
		temp_3=0;
	}
}


